# puppet-scvmm

Puppet module for Microsoft System Center Virtual Machine Manager (SCVMM)

[![Build Status](https://magnum.travis-ci.com/dell-asm/puppet-scvmm.png?token=LKiqZJU5Ck5Zd5swgkeS&branch=master)](https://magnum.travis-ci.com/dell-asm/puppet-scvmm)

## Usage

### Overview

The SCVMM module manages resource on Microsoft System Center Virtual Machine Manager via [Windows Remote Management (WinRM)](http://msdn.microsoft.com/en-us/library/aa384426.aspx). The transport module pattern is used to communicate with the SCVMM server (similar to VMware vCenter).

```
+-----------+         +---------+        +--------+
|           |  WinRM  |         | -----> | HyperV |
| Appliance | +-----> |  SCVMM  |        +--------+
|           |         |         | -----> | HyperV |
+-----------+         +---------+        +--------+
```

### WinRM

The transport connection setting depends on the SCVMM host configuration. The default configuration is basic auth which requires the following WinRM settings. This is adjustable, see [WinRM gem](https://github.com/WinRb/WinRM) for additional documentation.

```powershell
winrm set winrm/config/client/auth '@{Basic="true"}'
winrm set winrm/config/service/auth '@{Basic="true"}'
winrm set winrm/config/service '@{AllowUnencrypted="true"}'
```

WinRM processes are limited to 150 MB or memory by default. We need to increase this to 1024 to avoid out of memory errors:
```powershell
winrm set winrm/config/winrs '@{MaxMemoryPerShellMB=1024}'
```

There is a known issue with WMF 3.0. If the MaxMemoryPerShellMB configuration appears to be ignored, please see [KB2842230](http://support.microsoft.com/kb/2842230). The hotfix for Windows 8/Windows 2012 x64 is available at the following [link](http://hotfixv4.microsoft.com/Windows%208%20RTM/nosp/Fix452763/9200/free/463941_intl_x64_zip.exe).

### Transport

The transport resource specifies the connectivity information for the SCVMM server:
```puppet
$username = 'administrator'
$password = 'password'

transport { 'winrm':
  server   => '192.168.1.1',
  username => $username,
  password => $password,
}
```

The transport resource option parameter allows adjustment of WinRM connectivity type and connectivity options such as port (for more details see WinRM transport implementation):
```puppet
transport { 'winrm':
  server   => '192.168.1.1',
  username => $username,
  password => $password,
  options  => {
    'connectivity' => 'ssl',
    'port'         => '5986',
  }
}
```

### SCVMM Resources

Once connectivity is specified the scvmm module can manage the following resource on the scvmm server:
```puppet
sc_runasaccount { 'demo':
  ensure    => present,
  username  => $username,
  password  => $password,
  transport => Transport['winrm'],
}

scvm_host_group { 'All Hosts\demo':
  ensure    => present,
  transport => Transport['winrm'],
}

scvm_host_group { 'All Hosts\demo\subdir1':
  ensure    => present,
  transport => Transport['winrm'],
}

scvm_host { 'host':
  ensure             => present,
  path               => 'All Hosts\host',
  management_account => 'demo',
  username           => $username,
  password           => $password,
  transport          => Transport['winrm'],
}

scvm_host_cluster { 'cluster1':
  ensure            => present,
  hosts             => ['hosts'],
  path              => 'All Hosts\demo',
  username          => $username,
  password          => $password,
  transport         => Transport['winrm'],
}

sc_logical_network { 'vlan_tests':
  ensure                               => present,
  logical_network_definition_isolation => false,
  enable_network_virtualization        => true,
  use_gre                              => true,
  is_pvlan                             => false,
  transport                            => Transport['winrm'],
}
```
The scvm_host_group will establish automatic dependency to the parent host group. The example above 'All Hosts\demo\subdir1' will autorequire 'All Hosts\demo'. Similarly the scvm_host and scvm_host_cluster will autorequire the host_group specified in the path parameter.
