unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'
require 'json'

Puppet::Type.type(:sc_logical_network_definition).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    filter = '| Select Name,{$_.LogicalNetwork.Name},{$_.Hostgroups.Path},SubnetVLANS | ConvertTo-Json'
    result = vmm(exist_cmd + filter)
    begin 
      @result = PSON.parse(result)
    rescue PSON::ParserError
      @result = {}
    end
    @result = hashkey_downcase(@result)
    @result['name'] == resource[:name]
  end

  def result_map
    {
      '$_.LogicalNetwork.Name' => 'logical_network',
      '$_.Hostgroups.Path' => 'host_groups',
      'SubnetVLans' => 'subnet_vlans'
    }
  end

  def create
    command = logical_network_cmd + host_group_cmd + subnet_vlan_cmd + create_cmd
    vmm(command)
  end

  def destroy
    if resource[:skip_deletion]
      command = logical_network_cmd + update_host_group_cmd
      begin
        output = vmm(command)
      rescue Exception => ex
        Puppet.debug("Got error while unmapping the logical network definition: #{ex.inspect}")
        if ex.message.match(/with atleast one host group/)
          # Check if there is only one cluster in this host-group
          Puppet.debug('This is last host-group mapping. We can remove the logical network definition')
          if get_hostgroup_cluster <= 1
            Puppet.debug('There is only one cluster associated with the hostgroup')
            command = "#{exist_cmd} | Remove-SCLogicalNetworkDefinition"
            vmm(command)
          end
        end
      end
    else
      command = "#{exist_cmd} | Remove-SCLogicalNetworkDefinition"
      vmm(command)
    end

  end

  def get_hostgroup_cluster
    cmd = "Get-scvmhostcluster -vmmserver localhost | where hostgroup -like #{quote(resource[:host_groups][0])} | select name | ConvertTo-Json"
    Puppet.debug("Command for getting clusters information: #{cmd}")
    clusters = ( vmm(cmd) || [] )
    Puppet.debug("Clusters associated with host-group: #{clusters}")
    clusters.length == 0 ? clusters_length = 0 : clusters_length = JSON.parse(clusters).length
    clusters_length
  end

  def subnet_vlans
    subnets = Array.new
    @result['subnet_vlans'].each do |subnet|
      subnets.push({
        'subnet' => subnet['Subnet'],
        'vlan' => subnet['VLanID'].to_s,
      })
    end
    subnets
  end

  def subnet_vlans=(value)
    cmd = '$SubnetVLAN = @();'
    if resource[:inclusive] == :true
      vlans = resource[:subnet_vlans]
    else
      vlans = merge_subnets(subnet_vlans, resource[:subnet_vlans])
    end
    vlans.each do |subnet|
      if subnet['subnet'].size == 0
        cmd += "$SubnetVLAN += New-SCSubnetVLAN -VLAN #{subnet['vlan']} -VMMServer localhost;"
      else
        cmd += "$SubnetVLAN += New-SCSubnetVLAN -Subnet #{subnet['subnet']} -VLAN #{subnet['vlan']} -VMMServer localhost;"
      end
    end
    command = logical_network_cmd + cmd + update_cmd
    vmm(command)
  end

  def merge_subnets(old, new)
    existing_vlans = {}
    result = []
    old.each do |subnet|
      existing_vlans[subnet['vlan']] = subnet
    end

    new.each do |subnet|
      existing_vlans.delete subnet['vlan']
      result.push(subnet)
    end

    existing_vlans.each do |vlan, subnet|
      result.push(subnet)
    end
    result
  end

  def host_groups
    case @result['host_groups']
    when ::Hash
      @result['host_groups']['value']
    when ::Array
      @result['host_groups']
    when ::String
      [ @result['host_groups'] ]
    when nil
      []
    else
      raise Puppet::Error, "Unexpected result for host_groups: #{@result['host_groups']}"
    end
  end

  def host_groups=(value)
    command = logical_network_cmd + update_host_group_cmd
    vmm(command)
  end

  def exist_cmd
    ps_cmd( 'Get-SCLogicalNetworkDefinition',
      {
        :VMMServer => 'localhost',
        :Name => quote(resource[:name]),
      },
      :terminate => false,
    )
  end

  def logical_network_cmd
    ps_cmd( 'Get-SCLogicalNetwork',
      {
        :VMMServer => 'localhost',
        :Name => quote(resource[:logical_network]),
      },
      :variable => '$LogicalNetwork',
    )
  end

  def logical_network_definition_cmd
    ps_cmd( 'Get-SCLogicalNetworkDefinition',
      {
        :Name => quote(resource[:name]),
      },
      :variable => '$LogicalNetworkDefinition',
    )
  end

  def host_group_cmd
    cmd = '$HostGroup = @();'
    # We are only capable of additive and not removal at this point:
    hostgroups = resource[:host_groups] - host_groups
    hostgroups.each do |host_group|
      cmd += "$HostGroup += Get-SCVMHostGroup -VMMServer localhost | where { $_.Path -eq #{quote(host_group)} };"
    end
    cmd
  end

  def get_host_group_cmd
    host_groups = resource[:host_groups][0]
    host_groups.match(/\\/) ? host_group = host_groups.scan(/All Hosts\\(.*)/i).flatten.first : host_group = host_groups
    command = "$hostgroup = Get-SCVMHostGroup -Name #{host_group}; "
  end


  def subnet_vlan_cmd
    cmd = '$SubnetVLAN = @();'
    resource[:subnet_vlans].each do |subnet|
      if subnet['subnet'].size == 0
        cmd += "$SubnetVLAN += New-SCSubnetVLAN -VLAN #{subnet['vlan']};"
      else
        cmd += "$SubnetVLAN += New-SCSubnetVLAN -Subnet #{subnet['subnet']} -VLAN #{subnet['vlan']};"
      end
    end
    cmd
  end

  def create_cmd
    ps_cmd( 'New-SCLogicalNetworkDefinition',
      {
        :VMMServer => 'localhost',
        :Name => quote(resource[:name]),
        :LogicalNetwork => '$LogicalNetwork',
        :VMHostGroup => '$HostGroup',
        :SubnetVLAN => '$SubnetVLAN',
        :RunAsynchronouly => resource[:async],
      },
      :terminate => false,
    )
  end

  def update_cmd
    logicalnetworkdefinition = "$logicalnetworkdefinition = get-SCLogicalNetworkDefinition -name #{quote(resource[:name])} ;"
    command = ps_cmd( 'Set-SCLogicalNetworkDefinition',
      {
        :VMMServer => 'localhost',
        :Name => quote(resource[:name]),
        :LogicalNetworkDefinition => '$logicalnetworkdefinition',
        :SubnetVLAN => '$SubnetVLAN',
      }
    )
    command = logicalnetworkdefinition + command
  end

  def update_host_group_cmd
    logicalnetworkdefinition = "$logicalnetworkdefinition = get-SCLogicalNetworkDefinition -name #{quote(resource[:name])} ;"
    if resource[:ensure] == :absent
      command = ps_cmd( 'Set-SCLogicalNetworkDefinition',
        {
            :VMMServer => 'localhost',
            :Name => quote(resource[:name]),
            :LogicalNetworkDefinition => '$logicalnetworkdefinition',
            :RemoveVMHostGroup => '$HostGroup',
        }
      )
      command = logicalnetworkdefinition + get_host_group_cmd +  command
    else
      command = ps_cmd( 'Set-SCLogicalNetworkDefinition',
        {
            :VMMServer => 'localhost',
            :Name => quote(resource[:name]),
            :LogicalNetworkDefinition => '$logicalnetworkdefinition',
            :AddVMHostGroup => '$HostGroup',
        }
      )
      command = logicalnetworkdefinition + host_group_cmd +  command
    end

  end
end
