unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

vmware_module = Puppet::Module.find('vmware_lib', Puppet[:environment].to_s)
require File.join vmware_module.path, 'lib/puppet_x/vmware/util'

Puppet::Type.type(:scvm_host_cluster).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    @cluster = PSON.parse(vmm(exist_cmd+ '| Select Name, ClusterName, IPAddresses | ConvertTo-Json'))
    @cluster and (@cluster['ClusterName']==resource[:name] or @cluster['Name'] == resource[:name])
  rescue RuntimeError
    false
  end

  def create
    vmm(create_cmd)
  end

  def destroy
    vmm(destroy_cmd)
  end

  def hosts
    @hosts ||= get_existing_hosts
  end

  def get_existing_hosts
    # We can't grab cluster nodes directly due to ConvertTo-Json bug on Windows 2012r2:
    # ConvertTo-Json : The converted JSON string is in bad format.
    nodes = PSON.parse(vmm(exist_cmd + '| Select Nodes | ForEach-Object { echo $_.nodes} | Select name | ConvertTo-Json'))
    case nodes
    when ::Array
      nodes = nodes.collect{|n| n['Name']}
    when ::Hash
      [ nodes['Name'] ]
    else
      raise Puppet::Error, "Invalid cluster hosts data #{nodes}"
    end
  end

  def hosts=(values)
    new_hosts = values.select { |new| !hosts.any? { |old| old.casecmp(new) == 0 } }
    install_hosts(new_hosts) if !new_hosts.empty?
  end

  def host_exists?(host)
    result = vmm("Get-SCVMHost -ComputerName #{host} -VMMServer localhost | select Name")
    result and result.downcase.include?(host.downcase)
  rescue RuntimeError
    false
  end

  def add_host(host)
    cluster = "$cluster = #{exist_cmd};"
    command = ps_cmd('Add-SCVMHost',
      {
        :Name => quote(host),
        :VMMServer => 'localhost',
        :Credential => '$credential',
        :VMHostCluster => '$cluster',
        :RunAsynchronouly => resource[:async],
       }
     )
     command = credential + cluster + command
     vmm(command)
  end

  def install_hosts(hosts)
    # Need to verify host is part of SCVMM
    hosts.each do |host|
      raise(Puppet::Error, "Host: #{host} not part of SCVMM.") unless host_exists?(host)
    end

    hosts_cmd = "$hosts = (Get-SCVMhost -VMMServer 'localhost' | where {$_.Name -match #{quote(hosts.join('|'))} });"
    cluster_cmd = "$cluster = #{exist_cmd};"
    command = ps_cmd('Install-SCVMHostCluster',
      {
        :VMMServer => 'localhost',
        :Credential => '$credential',
        :VMHost => '$hosts',
        :VMHostCluster => '$cluster',
        :RunAsynchronously => resource[:async],
      }
    )
    command = credential + hosts_cmd + cluster_cmd + command
    vmm(command) unless hosts.empty?
  end

  def remove_hosts
    existing_hosts = get_existing_hosts
    Puppet.debug("Existing hosts: #{existing_hosts}")
    # Get the nodes which are in the existing hosts list
    existing_hosts.delete_if { |x| resource[:remove_hosts].include?(x)}
    Puppet.debug("Existing Hosts: #{existing_hosts}")
  end

  def remove_hosts=(value)
    hosts = []
    value.class == String ? hosts.push(value) : hosts = value
    hosts.each do |host|
      hostname = host.split('.')[0]
      cluster_fqdn = "#{resource[:name]}.#{resource[:fqdn]}"

      cluster_name = "#{resource[:name]}" + "." + "#{resource[:fqdn]}"
      cluster = "$cluster = (Get-SCVMHostCluster -name #{quote(cluster_name)} | select name);"
      hosts_info = "$hosts = (Get-SCVMhost -VMMServer 'localhost' | where {$_.Name -match #{quote(host)} });"
      import_details = "$importdetails = Get-VMMServer -ComputerName localhost;"
      remove_host_cmd = ps_cmd('Uninstall-SCVMHostCluster', { :VMHost => '$hosts',
                                                       :VMMServer => 'localhost',
                                                          },
                               :terminate => false)
      maintenance_mode_cmd = ps_cmd('Disable-SCVMHost',{ :VMHost => '$hosts'}, :terminate => false)
      command = cluster + hosts_info + maintenance_mode_cmd + ';' + remove_host_cmd
      Puppet.debug("Remove command: #{command}")
      vmm(command)
    end
    vmm(refresh_cluster_cmd)
  end

  def refresh_cluster_cmd()
    # Refresh host cluster
    cluster_name = "#{resource[:name]}" + "." + "#{resource[:fqdn]}"
    Puppet.debug "Refreshing host cluster"
    cluster = "$cluster = (Get-SCVMHostCluster -name #{quote(cluster_name)});"
    import_details = "$importdetails = Get-VMMServer -ComputerName localhost;"
    command = ps_cmd('Refresh-VMHostCluster',
                     {
                         :VMHostCluster => '$cluster',
                     }
    )
    command = credential + import_details + cluster + command
  end

  def exist_cmd
    if resource[:fqdn] and resource[:name] !~ /#{resource[:fqdn]}$/
      cluster_name = resource[:name] + "." + resource[:fqdn]
    else
      cluster_name = resource[:name]
    end

    ps_cmd('Get-SCVMHostCluster',
      {
        :VMMServer => 'localhost',
        :Name => quote(cluster_name),
      },
      :terminate => false,
    )
  end

  def create_cmd
    prefix = '$hostGroup = (' + ps_cmd('Get-SCVMHostGroup',
      {
        :VMMServer => 'localhost',
      },
      :terminate => false,
    ) + "| where { $_.path -eq #{quote(resource[:path])} });"

    host_search_pattern = []
    resource[:hosts].select {|x| host_search_pattern << "^#{x}"}
    hosts = "$hosts = (Get-SCVMhost -VMMServer 'localhost' | where {$_.Name -match #{quote(host_search_pattern.join('|'))} });"
    command = ps_cmd('Install-SCVMHostCluster',
      {
        :ClusterName => quote(resource[:name]),
        :VMMServer => 'localhost',
        :Credential => '$credential',
        :Description => quote(resource[:description]),
        :ClusterIPAddress => resource[:cluster_ipaddress],
        :VMhost => '$hosts',
        :RunAsynchronouly => resource[:async],
        #:VMHostGroup => resource[:path] ? '$hostGroup' : false,
      },
    )
    command = credential + hosts + command
  end

  def destroy_cmd
    cluster = ps_cmd('Get-SCVMHostCluster',
      {
        :VMMServer => 'localhost',
        :Name => quote(resource[:name]),
      },
      :variable => '$cluster',
    )
    command = ps_cmd('Uninstall-SCVMHostCluster',
      {
        :VMHostCluster => '$cluster',
        :RunAsynchronously => resource[:async],
      },
    )
    command = cluster + command
  end
end
